/**
 * @author Titus Wormer
 * @copyright 2015 Titus Wormer
 * @license MIT
 * @module no-emphasis-as-heading
 * @fileoverview
 *   Warn when emphasis (including strong), instead of a heading, introduces
 *   a paragraph.
 *
 * @example {"name": "valid.md"}
 *
 *   # Foo
 *
 *   Bar.
 *
 * @example {"name": "invalid.md", "label": "input"}
 *
 *   *Foo*
 *
 *   Bar.
 *
 *   __Qux__
 *
 *   Quux.
 *
 * @example {"name": "invalid.md", "label": "output"}
 *
 *   1:1-1:6: Don’t use emphasis to introduce a section, use a heading
 *   5:1-5:8: Don’t use emphasis to introduce a section, use a heading
 */

'use strict';

var rule = require('unified-lint-rule');
var visit = require('unist-util-visit');
var generated = require('unist-util-generated');

module.exports = rule('remark-lint:no-emphasis-as-heading', noEmphasisAsHeading);

function noEmphasisAsHeading(ast, file) {
  visit(ast, 'paragraph', visitor);

  function visitor(node, index, parent) {
    var children = node.children;
    var child = children[0];
    var prev = parent.children[index - 1];
    var next = parent.children[index + 1];

    if (generated(node)) {
      return;
    }

    if (
      (!prev || prev.type !== 'heading') &&
      next &&
      next.type === 'paragraph' &&
      children.length === 1 &&
      (child.type === 'emphasis' || child.type === 'strong')
    ) {
      file.message('Don’t use emphasis to introduce a section, use a heading', node);
    }
  }
}
